@@ -26,38 +26,44 @@
 #include "../../deadbeef.h"
 #include "../../strdupa.h"
 
-#if !FFMPEG_OLD
-
 #include <libavformat/avformat.h>
 #include <libavcodec/avcodec.h>
 #include <libavutil/avutil.h>
 #include <libavutil/avstring.h>
 
-#else
 
-#include <ffmpeg/avformat.h>
-#include <ffmpeg/avcodec.h>
-#include <ffmpeg/avutil.h>
-#include <ffmpeg/avstring.h>
-
-#define AVERROR_EOF AVERROR(EPIPE)
-
-#if LIBAVFORMAT_VERSION_MAJOR < 53
-#define av_register_protocol register_protocol
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(54, 6, 0)
+#define avformat_find_stream_info(ctx,data) av_find_stream_info(ctx)
+#define avcodec_open2(ctx,codec,data) avcodec_open(ctx,codec)
 #endif
 
-#ifndef AV_VERSION_INT
-#define AV_VERSION_INT(a, b, c) (a<<16 | b<<8 | c)
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 17, 0)
+#define avformat_free_context(ctx) av_close_input_file(ctx)
 #endif
 
+#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(55, 28, 0)
+#    define avcodec_free_frame(frame) av_frame_free(frame)
+#elif LIBAVCODEC_VERSION_INT > AV_VERSION_INT(54, 59, 100)
+     // has avcodec_free_frame
+#elif LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 40, 0)
+#    define avcodec_free_frame(frame) av_freep(frame) // newest -- av_freep
 #endif
 
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
-#define AVCODEC_MAX_AUDIO_FRAME_SIZE 192000
-#define av_find_stream_info(ctx) avformat_find_stream_info(ctx,NULL)
-#define avcodec_open(ctx,codec) avcodec_open2(ctx,codec,NULL)
-#define av_get_bits_per_sample_format(fmt) (av_get_bytes_per_sample(fmt)*8)
-#define av_close_input_file(ctx) avformat_close_input(&(ctx))
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(57, 8, 0)
+#define av_packet_unref(packet) av_free_packet(packet)
+#endif
+
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52, 64, 0)
+#define AVMEDIA_TYPE_AUDIO CODEC_TYPE_AUDIO
+#endif
+
+
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(55, 28, 1)
+#define av_frame_alloc() avcodec_alloc_frame()
+#endif
+
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 2, 0)
+#define avformat_open_input(ctx, uri, fmt, options) av_open_input_file(ctx, uri, NULL, fmt, options)
 #endif
 
 //#define trace(...) { fprintf(stderr, __VA_ARGS__); }
@@ -69,13 +75,17 @@
 static DB_decoder_t plugin;
 static DB_functions_t *deadbeef;
 
-#define DEFAULT_EXTS "aa3;oma;ac3;vqf;amr;opus;tak"
+#define DEFAULT_EXTS "aa3;oma;ac3;vqf;amr;tak;dsf;dff;wma;3gp;mp4;m4a"
+#define UNPOPULATED_EXTS_BY_FFMPEG \
+    "aif,aiff,afc,aifc,amr,asf," \
+    "wmv,wma,au,caf,webm," \
+    "gxf,lbc,mmf,mpg,mpeg,ts,m2t," \
+    "m2ts,mts,mxf,rm,ra,roq,sox," \
+    "spdif,swf,rcv,voc,w64,wav,wv"
 
-#define EXT_MAX 100
+#define EXT_MAX 1024
 
-#define FFMPEG_MAX_ANALYZE_DURATION 500000
-
-static char * exts[EXT_MAX] = {NULL};
+static char * exts[EXT_MAX+1] = {NULL};
 
 enum {
     FT_ALAC = 0,
@@ -87,10 +97,6 @@ enum {
     FT_UNKNOWN = 5
 };
 
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(54, 6, 0)
-#define FF_PROTOCOL_NAME "deadbeef"
-#endif
-
 typedef struct {
     DB_fileinfo_t info;
     AVCodec *codec;
@@ -105,17 +111,15 @@ typedef struct {
     int left_in_packet;
     int have_packet;
 
-    char *buffer; // must be AVCODEC_MAX_AUDIO_FRAME_SIZE
+    char *buffer;
     int left_in_buffer;
+    int buffer_size;
 
-    int startsample;
-    int endsample;
-    int currentsample;
+    int64_t startsample;
+    int64_t endsample;
+    int64_t currentsample;
 } ffmpeg_info_t;
 
-static DB_playItem_t *current_track;
-static DB_fileinfo_t *current_info;
-
 static DB_fileinfo_t *
 ffmpeg_open (uint32_t hints) {
     DB_fileinfo_t *_info = malloc (sizeof (ffmpeg_info_t));
@@ -123,8 +127,34 @@ ffmpeg_open (uint32_t hints) {
     return _info;
 }
 
+// ensure that the buffer can contain entire frame of frame_size bytes per channel
+static int
+ensure_buffer (ffmpeg_info_t *info, int frame_size) {
+    if (!info->buffer || info->buffer_size < frame_size * info->ctx->channels) {
+        if (info->buffer) {
+            free (info->buffer);
+            info->buffer = NULL;
+        }
+        info->buffer_size = frame_size*info->ctx->channels;
+        info->left_in_buffer = 0;
+        int err = posix_memalign ((void **)&info->buffer, 16, info->buffer_size);
+        if (err) {
+            fprintf (stderr, "ffmpeg: failed to allocate %d bytes of buffer memory\n", info->buffer_size);
+            return -1;
+        }
+    }
+    return 0;
+}
+
 static int
 ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
+    // Don't allow playing network streams.
+    // Even when ffmpeg has network support, it's causing too many problems this way.
+    const char *fname = deadbeef->pl_find_meta (it, ":URI");
+    if (!deadbeef->is_local_file (fname)) {
+        return -1;
+    }
+
     ffmpeg_info_t *info = (ffmpeg_info_t *)_info;
     trace ("ffmpeg init %s\n", deadbeef->pl_find_meta (it, ":URI"));
     // prepare to decode the track
@@ -137,54 +167,30 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     deadbeef->pl_lock ();
     {
         const char *fname = deadbeef->pl_find_meta (it, ":URI");
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
         uri = strdupa (fname);
-#else
-        int l = strlen (fname);
-        uri = alloca (l + sizeof (FF_PROTOCOL_NAME) + 1);
-
-        // construct uri
-        memcpy (uri, FF_PROTOCOL_NAME, sizeof (FF_PROTOCOL_NAME)-1);
-        memcpy (uri + sizeof (FF_PROTOCOL_NAME)-1, ":", 1);
-        memcpy (uri + sizeof (FF_PROTOCOL_NAME), fname, l);
-        uri[sizeof (FF_PROTOCOL_NAME) + l] = 0;
-#endif
     }
     deadbeef->pl_unlock ();
     trace ("ffmpeg: uri: %s\n", uri);
 
     // open file
     trace ("\033[0;31mffmpeg av_open_input_file\033[37;0m\n");
-    current_track = it;
-    current_info = _info;
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
-    avformat_network_init();
+
     info->fctx = avformat_alloc_context ();
+
     if ((ret = avformat_open_input(&info->fctx, uri, NULL, NULL)) < 0) {
-#else
-    if ((ret = av_open_input_file(&info->fctx, uri, NULL, 0, NULL)) < 0) {
-#endif
-        current_track = NULL;
         trace ("\033[0;31minfo->fctx is %p, ret %d/%s\033[0;31m\n", info->fctx, ret, strerror(-ret));
         return -1;
     }
     trace ("\033[0;31mav_open_input_file done, ret=%d\033[0;31m\n", ret);
-    current_track = NULL;
-    current_info = NULL;
 
-    trace ("\033[0;31mffmpeg av_find_stream_info\033[37;0m\n");
+    trace ("\033[0;31mffmpeg avformat_find_stream_info\033[37;0m\n");
     info->stream_id = -1;
-    info->fctx->max_analyze_duration = FFMPEG_MAX_ANALYZE_DURATION;
-    av_find_stream_info(info->fctx);
+    info->fctx->max_analyze_duration = AV_TIME_BASE;
+    avformat_find_stream_info(info->fctx, NULL);
     for (i = 0; i < info->fctx->nb_streams; i++)
     {
         info->ctx = info->fctx->streams[i]->codec;
-        if (info->ctx->codec_type ==
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52, 64, 0)
-            AVMEDIA_TYPE_AUDIO)
-#else
-            CODEC_TYPE_AUDIO)
-#endif
+        if (info->ctx->codec_type == AVMEDIA_TYPE_AUDIO)
         {
             info->codec = avcodec_find_decoder (info->ctx->codec_id);
             if (info->codec != NULL) {
@@ -202,36 +208,28 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     trace ("ffmpeg can decode %s\n", deadbeef->pl_find_meta (it, ":URI"));
     trace ("ffmpeg: codec=%s, stream=%d\n", info->codec->name, i);
 
-    if (avcodec_open (info->ctx, info->codec) < 0) {
-        trace ("ffmpeg: avcodec_open failed\n");
+    if (avcodec_open2 (info->ctx, info->codec, NULL) < 0) {
+        trace ("ffmpeg: avcodec_open2 failed\n");
         return -1;
     }
 
     deadbeef->pl_replace_meta (it, "!FILETYPE", info->codec->name);
 
-    int bps = av_get_bits_per_sample_format (info->ctx->sample_fmt);
+    int bps = av_get_bytes_per_sample (info->ctx->sample_fmt)*8;
     int samplerate = info->ctx->sample_rate;
-    float duration = info->fctx->duration / (float)AV_TIME_BASE;
-    trace ("ffmpeg: bits per sample is %d\n", bps);
-    trace ("ffmpeg: samplerate is %d\n", samplerate);
-    trace ("ffmpeg: duration is %lld/%fsec\n", info->fctx->duration, duration);
 
-    int totalsamples = info->fctx->duration * samplerate / AV_TIME_BASE;
+    if (bps <= 0 || info->ctx->channels <= 0 || samplerate <= 0) {
+        return -1;
+    }
+
+    int64_t totalsamples = info->fctx->duration * samplerate / AV_TIME_BASE;
     info->left_in_packet = 0;
     info->left_in_buffer = 0;
 
     memset (&info->pkt, 0, sizeof (info->pkt));
     info->have_packet = 0;
 
-    int err = posix_memalign ((void **)&info->buffer, 16, AVCODEC_MAX_AUDIO_FRAME_SIZE);
-    if (err) {
-        fprintf (stderr, "ffmpeg: failed to allocate buffer memory\n");
-        return -1;
-    }
-
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 40, 0)
-    info->frame = avcodec_alloc_frame();
-#endif
+    info->frame = av_frame_alloc();
 
     // fill in mandatory plugin fields
     _info->plugin = &plugin;
@@ -243,22 +241,19 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
         _info->fmt.is_float = 1;
     }
 
+    // FIXME: channel layout from ffmpeg
+    // int64_t layout = info->ctx->channel_layout;
 
-    int64_t layout = info->ctx->channel_layout;
-    if (layout != 0, 0) {
-        _info->fmt.channelmask = layout;
-    }
-    else {
-        for (int i = 0; i < _info->fmt.channels; i++) {
-            _info->fmt.channelmask |= 1 << i;
-        }
+    for (int i = 0; i < _info->fmt.channels; i++) {
+        _info->fmt.channelmask |= 1 << i;
     }
 
     // subtrack info
     info->currentsample = 0;
-    if (it->endsample > 0) {
-        info->startsample = it->startsample;
-        info->endsample = it->endsample;
+    int64_t endsample = deadbeef->pl_item_get_endsample (it);
+    if (endsample > 0) {
+        info->startsample = deadbeef->pl_item_get_startsample (it);
+        info->endsample = endsample;
         plugin.seek_sample (_info, 0);
     }
     else {
@@ -273,27 +268,21 @@ ffmpeg_free (DB_fileinfo_t *_info) {
     trace ("ffmpeg: free\n");
     ffmpeg_info_t *info = (ffmpeg_info_t*)_info;
     if (info) {
-#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(54, 59, 100)
         if (info->frame) {
             avcodec_free_frame(&info->frame);
         }
-#elif LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 40, 0)
-        if (info->frame) {
-            av_freep (&info->frame);
-        }
-#endif
         if (info->buffer) {
             free (info->buffer);
         }
         // free everything allocated in _init and _read
         if (info->have_packet) {
-            av_free_packet (&info->pkt);
+            av_packet_unref (&info->pkt);
         }
         if (info->ctx) {
             avcodec_close (info->ctx);
         }
         if (info->fctx) {
-            av_close_input_file (info->fctx);
+            avformat_free_context (info->fctx);
         }
         free (info);
     }
@@ -306,14 +295,13 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
 
     _info->fmt.channels = info->ctx->channels;
     _info->fmt.samplerate = info->ctx->sample_rate;
-    _info->fmt.bps = av_get_bits_per_sample_format (info->ctx->sample_fmt);
+    _info->fmt.bps = av_get_bytes_per_sample (info->ctx->sample_fmt) * 8;
     _info->fmt.is_float = (info->ctx->sample_fmt == AV_SAMPLE_FMT_FLT || info->ctx->sample_fmt == AV_SAMPLE_FMT_FLTP);
 
     int samplesize = _info->fmt.channels * _info->fmt.bps / 8;
 
     if (info->endsample >= 0 && info->currentsample + size / samplesize > info->endsample) {
-        size = (info->endsample - info->currentsample + 1) * samplesize;
-        if (size <= 0) {
+        if ((info->endsample - info->currentsample + 1) * samplesize <= 0) {
             return 0;
         }
     }
@@ -341,7 +329,7 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
         }
 
         while (info->left_in_packet > 0 && size > 0) {
-            int out_size = AVCODEC_MAX_AUDIO_FRAME_SIZE;
+            int out_size = info->buffer_size;
             int len;
             //trace ("in: out_size=%d(%d), size=%d\n", out_size, AVCODEC_MAX_AUDIO_FRAME_SIZE, size);
 
@@ -349,6 +337,9 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
             int got_frame = 0;
             len = avcodec_decode_audio4(info->ctx, info->frame, &got_frame, &info->pkt);
             if (len > 0) {
+                if (ensure_buffer (info, info->frame->nb_samples * (_info->fmt.bps >> 3))) {
+                    return -1;
+                }
                 if (av_sample_fmt_is_planar(info->ctx->sample_fmt)) {
                     out_size = 0;
                     for (int c = 0; c < info->ctx->channels; c++) {
@@ -380,10 +371,15 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
                 }
             }
 
-#elif LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52,25,0)
+#else
+            if (ensure_buffer (info, 16384)) { // FIXME: how to get the packet size in old ffmpeg?
+                return -1;
+            }
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52,25,0)
             len = avcodec_decode_audio3 (info->ctx, (int16_t *)info->buffer, &out_size, &info->pkt);
 #else
             len = avcodec_decode_audio2 (info->ctx, (int16_t *)info->buffer, &out_size, info->pkt.data, info->pkt.size);
+#endif
 #endif
             trace ("out: out_size=%d, len=%d\n", out_size, len);
             if (len <= 0) {
@@ -400,7 +396,7 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
 
         // read next packet
         if (info->have_packet) {
-            av_free_packet (&info->pkt);
+            av_packet_unref (&info->pkt);
             info->have_packet = 0;
         }
         int errcount = 0;
@@ -432,7 +428,7 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
             }
             //trace ("idx:%d, stream:%d\n", info->pkt.stream_index, info->stream_id);
             if (info->pkt.stream_index != info->stream_id) {
-                av_free_packet (&info->pkt);
+                av_packet_unref (&info->pkt);
                 continue;
             }
             //trace ("got packet: size=%d\n", info->pkt.size);
@@ -468,7 +464,7 @@ ffmpeg_seek_sample (DB_fileinfo_t *_info, int sample) {
     // return 0 on success
     // return -1 on failure
     if (info->have_packet) {
-        av_free_packet (&info->pkt);
+        av_packet_unref (&info->pkt);
         info->have_packet = 0;
     }
     sample += info->startsample;
@@ -489,10 +485,6 @@ ffmpeg_seek_sample (DB_fileinfo_t *_info, int sample) {
 
 static int
 ffmpeg_seek (DB_fileinfo_t *_info, float time) {
-    ffmpeg_info_t *info = (ffmpeg_info_t*)_info;
-    // seek to specified time in seconds
-    // return 0 on success
-    // return -1 on failure
     return ffmpeg_seek_sample (_info, time * _info->fmt.samplerate);
 }
 
@@ -501,6 +493,7 @@ static const char *map[] = {
     "title", "title",
     "album", "album",
     "track", "track",
+    "tracktotal", "numtracks",
     "date", "year",
     "WM/Year", "year",
     "genre", "genre",
@@ -511,12 +504,42 @@ static const char *map[] = {
     "encoder", "encoder",
     "encoded_by", "vendor",
     "disc", "disc",
+    "disctotal", "numdiscs",
     "copyright", "copyright",
-    "tracktotal", "numtracks",
     "publisher", "publisher",
+    "originaldate","original_release_time",
+    "originalyear","original_release_year",
+    "WM/OriginalReleaseTime","original_release_time",
+    "WM/OriginalReleaseYear","original_release_year",
     NULL
 };
 
+static int
+ff_add_disc_meta (DB_playItem_t *it, const char *disc) {
+    char *slash = strchr (disc, '/');
+    if (slash) {
+        // split into disc/number
+        *slash = 0;
+        slash++;
+        deadbeef->pl_add_meta (it, "numdiscs", slash);
+    }
+    deadbeef->pl_add_meta (it, "disc", disc);
+    return 0;
+}
+
+static int
+ff_add_track_meta (DB_playItem_t *it, const char *track) {
+    char *slash = strchr (track, '/');
+    if (slash) {
+        // split into track/number
+        *slash = 0;
+        slash++;
+        deadbeef->pl_add_meta (it, "numtracks", slash);
+    }
+    deadbeef->pl_add_meta (it, "track", track);
+    return 0;
+}
+
 static int
 ffmpeg_read_metadata_internal (DB_playItem_t *it, AVFormatContext *fctx) {
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52,43,0)
@@ -547,8 +570,17 @@ ffmpeg_read_metadata_internal (DB_playItem_t *it, AVFormatContext *fctx) {
         do {
             tag = av_metadata_get (md, map[m], tag, AV_METADATA_DONT_STRDUP_KEY | AV_METADATA_DONT_STRDUP_VAL);
             if (tag) {
-                deadbeef->pl_append_meta (it, map[m+1], tag->value);
+                if (!strcmp (map[m+1], "disc")) {
+                    ff_add_disc_meta (it, tag->value);
+                }
+                else if (!strcmp (map[m+1], "track")) {
+                    ff_add_track_meta (it, tag->value);
+                }
+                else {
+                    deadbeef->pl_append_meta (it, map[m+1], tag->value);
+                }
             }
+
         } while (tag);
     }
 #else
@@ -560,7 +592,7 @@ ffmpeg_read_metadata_internal (DB_playItem_t *it, AVFormatContext *fctx) {
         if (!md) {
             continue;
         }
-        while (t = av_dict_get (md, "", t, AV_DICT_IGNORE_SUFFIX)) {
+        while ((t = av_dict_get (md, "", t, AV_DICT_IGNORE_SUFFIX))) {
             if (!strcasecmp (t->key, "replaygain_album_gain")) {
                 deadbeef->pl_set_item_replaygain (it, DDB_REPLAYGAIN_ALBUMGAIN, atof (t->value));
                 continue;
@@ -580,7 +612,15 @@ ffmpeg_read_metadata_internal (DB_playItem_t *it, AVFormatContext *fctx) {
 
             for (m = 0; map[m]; m += 2) {
                 if (!strcasecmp (t->key, map[m])) {
-                    deadbeef->pl_append_meta (it, map[m+1], t->value);
+                    if (!strcmp (map[m+1], "disc")) {
+                        ff_add_disc_meta (it, t->value);
+                    }
+                    else if (!strcmp (map[m+1], "track")) {
+                        ff_add_track_meta (it, t->value);
+                    }
+                    else {
+                        deadbeef->pl_append_meta (it, map[m+1], t->value);
+                    }
                     break;
                 }
             }
@@ -594,6 +634,18 @@ ffmpeg_read_metadata_internal (DB_playItem_t *it, AVFormatContext *fctx) {
     return 0;
 }
 
+static void
+print_error(const char *filename, int err)
+{
+    char errbuf[128];
+    const char *errbuf_ptr = errbuf;
+
+    if (av_strerror(err, errbuf, sizeof(errbuf)) < 0)
+        errbuf_ptr = strerror(AVUNERROR(err));
+    fprintf (stderr, "%s: %s\n", filename, errbuf_ptr);
+}
+
+
 static DB_playItem_t *
 ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     trace ("ffmpeg_insert %s\n", fname);
@@ -607,7 +659,6 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     AVCodecContext *ctx = NULL;
     AVFormatContext *fctx = NULL;
     int ret;
-    int l = strlen (fname);
     char *uri = NULL;
     int i;
 
@@ -625,27 +676,20 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
 
     // open file
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
-    avformat_network_init();
     fctx = avformat_alloc_context ();
+    fctx->max_analyze_duration = AV_TIME_BASE;
     if ((ret = avformat_open_input(&fctx, uri, NULL, NULL)) < 0) {
 #else
     if ((ret = av_open_input_file(&fctx, uri, NULL, 0, NULL)) < 0) {
 #endif
-        trace ("fctx is %p, ret %d/%s", fctx, ret, strerror(-ret));
+        print_error (uri, ret);
         return NULL;
     }
 
     trace ("fctx is %p, ret %d/%s\n", fctx, ret, strerror(-ret));
-    fctx->max_analyze_duration = FFMPEG_MAX_ANALYZE_DURATION;
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
-    int nb_streams = fctx->nb_streams;
     ret = avformat_find_stream_info(fctx, NULL);
-#else
-    ret = av_find_stream_info(fctx);
-    int nb_streams = fctx->nb_streams;
-#endif
     if (ret < 0) {
-        trace ("av_find_stream_info ret: %d/%s\n", ret, strerror(-ret));
+        trace ("avformat_find_stream_info ret: %d/%s\n", ret, strerror(-ret));
     }
     trace ("nb_streams=%x\n", nb_streams);
     for (i = 0; i < fctx->nb_streams; i++)
@@ -672,19 +716,19 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     if (codec == NULL)
     {
         trace ("ffmpeg can't decode %s\n", fname);
-        av_close_input_file(fctx);
+        avformat_free_context(fctx);
         return NULL;
     }
     trace ("ffmpeg can decode %s\n", fname);
     trace ("ffmpeg: codec=%s, stream=%d\n", codec->name, i);
 
-    if (avcodec_open (ctx, codec) < 0) {
-        trace ("ffmpeg: avcodec_open failed\n");
-        av_close_input_file(fctx);
+    if (avcodec_open2 (ctx, codec, NULL) < 0) {
+        trace ("ffmpeg: avcodec_open2 failed\n");
+        avformat_free_context(fctx);
         return NULL;
     }
 
-    int bps = av_get_bits_per_sample_format (ctx->sample_fmt);
+    int bps = av_get_bytes_per_sample (ctx->sample_fmt) * 8;
     int samplerate = ctx->sample_rate;
     float duration = fctx->duration / (float)AV_TIME_BASE;
 //    float duration = stream->duration * stream->time_base.num / (float)stream->time_base.den;
@@ -692,7 +736,11 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     trace ("ffmpeg: samplerate is %d\n", samplerate);
     trace ("ffmpeg: duration is %f\n", duration);
 
-    int totalsamples = fctx->duration * samplerate / AV_TIME_BASE;
+    if (bps <= 0 || ctx->channels <= 0 || samplerate <= 0) {
+        return NULL;
+    }
+
+    int64_t totalsamples = fctx->duration * samplerate / AV_TIME_BASE;
 
     DB_playItem_t *it = deadbeef->pl_item_alloc_init (fname, plugin.plugin.id);
     deadbeef->pl_replace_meta (it, ":FILETYPE", codec->name);
@@ -721,7 +769,7 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
         char s[100];
         snprintf (s, sizeof (s), "%lld", fsize);
         deadbeef->pl_add_meta (it, ":FILE_SIZE", s);
-        snprintf (s, sizeof (s), "%d", av_get_bits_per_sample_format (ctx->sample_fmt));
+        snprintf (s, sizeof (s), "%d", bps);
         deadbeef->pl_add_meta (it, ":BPS", s);
         snprintf (s, sizeof (s), "%d", ctx->channels);
         deadbeef->pl_add_meta (it, ":CHANNELS", s);
@@ -734,132 +782,116 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
 
     // free decoder
     avcodec_close (ctx);
-    av_close_input_file(fctx);
+    avformat_free_context(fctx);
 
-    // external cuesheet
-    DB_playItem_t *cue = deadbeef->plt_insert_cue (plt, after, it, totalsamples, samplerate);
+    DB_playItem_t *cue = deadbeef->plt_process_cue (plt, after, it, totalsamples, samplerate);
     if (cue) {
         deadbeef->pl_item_unref (it);
-        deadbeef->pl_item_unref (cue);
         return cue;
     }
+
     // now the track is ready, insert into playlist
     after = deadbeef->plt_insert_item (plt, after, it);
     deadbeef->pl_item_unref (it);
     return after;
 }
 
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(54, 6, 0)
-// vfs wrapper for ffmpeg, can't only be done with ffmpeg<0.11
 static int
-ffmpeg_vfs_open(URLContext *h, const char *filename, int flags)
-{
-    DB_FILE *f;
-    av_strstart(filename, FF_PROTOCOL_NAME ":", &filename);
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(54, 6, 0)
-    if (flags & URL_WRONLY) {
-        return -ENOENT;
-    }
-    else
-#endif
-    {
-        f = deadbeef->fopen (filename);
-    }
-
-    if (f == NULL)
-        return -ENOENT;
-
-    if (f->vfs->is_streaming ()) {
-        deadbeef->fset_track (f, current_track);
-        if (current_info) {
-            current_info->file = f;
+assign_new_ext (int n, const char* new_ext, size_t size) {
+    char* ext = malloc (size + 1);
+    strncpy (ext, new_ext, size);
+    for (int i = 0; i < n; i++) {
+        if (strcmp (exts[i], ext) == 0) {
+            free(ext);
+            return n;
         }
     }
-
-    h->priv_data = f;
-    return 0;
+    ext[size] = '\0';
+    free (exts[n]);
+    exts[n] = ext;
+    return n + 1;
 }
 
 static int
-ffmpeg_vfs_read(URLContext *h, unsigned char *buf, int size)
-{
-    trace ("ffmpeg_vfs_read %d\n", size);
-    int res = deadbeef->fread (buf, 1, size, h->priv_data);
-    return res;
-}
-
-static int
-ffmpeg_vfs_write(URLContext *h, const unsigned char *buf, int size)
-{
-    return -1;
-}
-
-static int64_t
-ffmpeg_vfs_seek(URLContext *h, int64_t pos, int whence)
-{
-    trace ("ffmpeg_vfs_seek %d %d\n", pos, whence);
-    DB_FILE *f = h->priv_data;
-
-    if (whence == AVSEEK_SIZE) {
-        return f->vfs->is_streaming () ? -1 : deadbeef->fgetlength (h->priv_data);
-    }
-    else if (f->vfs->is_streaming ()) {
-        return -1;
-    }
-    else {
-        int ret = deadbeef->fseek (h->priv_data, pos, whence);
-        return ret;
-    }
-}
-
-static int
-ffmpeg_vfs_close(URLContext *h)
-{
-    trace ("ffmpeg_vfs_close\n");
-    deadbeef->fclose (h->priv_data);
-    return 0;
-}
-
-static URLProtocol vfswrapper = {
-    .name = FF_PROTOCOL_NAME,
-    .url_open = ffmpeg_vfs_open,
-    .url_read = ffmpeg_vfs_read,
-    .url_write = ffmpeg_vfs_write,
-    .url_seek = ffmpeg_vfs_seek,
-    .url_close = ffmpeg_vfs_close,
-};
-#endif
-
-static void
-ffmpeg_init_exts (void) {
-    deadbeef->conf_lock ();
-    const char *new_exts = deadbeef->conf_get_str_fast ("ffmpeg.extensions", DEFAULT_EXTS);
-    for (int i = 0; exts[i]; i++) {
-        free (exts[i]);
-    }
-    exts[0] = NULL;
-
-    int n = 0;
+add_new_exts (int n, const char* new_exts, char delim) {
     while (*new_exts) {
         if (n >= EXT_MAX) {
             fprintf (stderr, "ffmpeg: too many extensions, max is %d\n", EXT_MAX);
             break;
         }
         const char *e = new_exts;
-        while (*e && *e != ';') {
+        while (*e && (*e != delim || *e == ' ')) {
             e++;
         }
         if (e != new_exts) {
-            char *ext = malloc (e-new_exts+1);
-            memcpy (ext, new_exts, e-new_exts);
-            ext[e-new_exts] = 0;
-            exts[n++] = ext;
+            n = assign_new_ext (n, new_exts, e-new_exts);
         }
         if (*e == 0) {
             break;
         }
         new_exts = e+1;
     }
+    return n;
+}
+
+static void
+ffmpeg_init_exts (void) {
+    deadbeef->conf_lock ();
+    const char *new_exts = deadbeef->conf_get_str_fast ("ffmpeg.extensions", DEFAULT_EXTS);
+    int use_all_ext = deadbeef->conf_get_int ("ffmpeg.enable_all_exts", 0);
+    for (int i = 0; exts[i]; i++) {
+        free (exts[i]);
+        exts[i] = NULL;
+    }
+    exts[0] = NULL;
+
+    int n = 0;
+    if (!use_all_ext) {
+        n = add_new_exts (n, new_exts, ';');
+    }
+	else {
+        AVInputFormat *ifmt  = NULL;
+        /*
+          * It's quite complicated to enumerate all supported extensions in
+         * ffmpeg. If a decoder defines extensions in ffmpeg, the probing
+         * mechanisim is disabled (see comments in avformat.h).
+         * Thus some decoders doesn't claim its extensions (e.g. WavPack) 
+         *
+         * To get these missing extensions, we need to search corresponding
+         * encoders for the same format, which will provide extensions for
+         * encoding purpose, because ffmpeg will guess the output format from
+         * the file name specified by users.
+         */
+        while ((ifmt = av_iformat_next(ifmt))) {
+#ifdef AV_IS_INPUT_DEVICE
+            if (ifmt->priv_class && AV_IS_INPUT_DEVICE(ifmt->priv_class->category))
+                continue; // Skip all input devices
+#endif
+
+            if (ifmt->flags & AVFMT_NOFILE)
+                continue; // Skip format that's not even a file
+
+#ifdef AV_CODEC_ID_FIRST_AUDIO
+            if (ifmt->raw_codec_id > 0 &&
+                    (ifmt->raw_codec_id < AV_CODEC_ID_FIRST_AUDIO || ifmt->raw_codec_id > AV_CODEC_ID_FIRST_SUBTITLE)
+               )
+                continue; // Skip all non-audio raw formats
+#endif
+            if (ifmt->long_name && strstr(ifmt->long_name, "subtitle"))
+                continue; // Skip all subtitle formats
+            if (ifmt->extensions)
+                n = add_new_exts (n, ifmt->extensions, ',');
+        }
+        /*
+          * The above code doesn't guarntee all extensions are
+         * included, however. In the portable build the encoders are disabled,
+         * thus some extensions cannot be retrived.
+         *
+         * To fix this, we need to add some known extensions in addition to
+         * scanned extensions.
+         */
+        n = add_new_exts (n, UNPOPULATED_EXTS_BY_FFMPEG, ',');
+    }
     exts[n] = NULL;
     deadbeef->conf_unlock ();
 }
@@ -881,13 +913,6 @@ ffmpeg_start (void) {
     avcodec_register_all ();
 #endif
     av_register_all ();
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
-#warning FFMPEG-0.11 and higher does not expose register_protocol API, which means that it cant work with MMS and HTTP plugins. If you need this functionality, please downgrade FFMPEG to version 0.10 or less, and rebuild the FFMPEG plugin
-#elif LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 69, 0)
-    av_register_protocol2 (&vfswrapper, sizeof(vfswrapper));
-#else
-    av_register_protocol (&vfswrapper);
-#endif
     return 0;
 }
 
@@ -930,27 +955,16 @@ ffmpeg_read_metadata (DB_playItem_t *it) {
     trace ("ffmpeg: uri: %s\n", uri);
 
     // open file
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
-    avformat_network_init();
-    fctx = avformat_alloc_context ();
     if ((ret = avformat_open_input(&fctx, uri, NULL, NULL)) < 0) {
-#else
-    if ((ret = av_open_input_file(&fctx, uri, NULL, 0, NULL)) < 0) {
-#endif
         trace ("fctx is %p, ret %d/%s", fctx, ret, strerror(-ret));
         return -1;
     }
 
-    av_find_stream_info(fctx);
+    avformat_find_stream_info(fctx, NULL);
     for (i = 0; i < fctx->nb_streams; i++)
     {
         ctx = fctx->streams[i]->codec;
-        if (ctx->codec_type ==
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52, 64, 0)
-            AVMEDIA_TYPE_AUDIO)
-#else
-            CODEC_TYPE_AUDIO)
-#endif
+        if (ctx->codec_type == AVMEDIA_TYPE_AUDIO)
         {
             codec = avcodec_find_decoder(ctx->codec_id);
             if (codec != NULL)
@@ -960,30 +974,31 @@ ffmpeg_read_metadata (DB_playItem_t *it) {
     if (codec == NULL)
     {
         trace ("ffmpeg can't decode %s\n", deadbeef->pl_find_meta (it, ":URI"));
-        av_close_input_file(fctx);
+        avformat_free_context(fctx);
         return -1;
     }
-    if (avcodec_open (ctx, codec) < 0) {
-        trace ("ffmpeg: avcodec_open failed\n");
-        av_close_input_file(fctx);
+    if (avcodec_open2 (ctx, codec, NULL) < 0) {
+        trace ("ffmpeg: avcodec_open2 failed\n");
+        avformat_free_context(fctx);
         return -1;
     }
 
     deadbeef->pl_delete_all_meta (it);
     ffmpeg_read_metadata_internal (it, fctx);
 
-    av_close_input_file(fctx);
+    avformat_free_context(fctx);
+
     return 0;
 }
 
 static const char settings_dlg[] =
+    "property \"Use all extensions supported by ffmpeg\" checkbox ffmpeg.enable_all_exts 0;\n"
     "property \"File Extensions (separate with ';')\" entry ffmpeg.extensions \"" DEFAULT_EXTS "\";\n"
 ;
 
 // define plugin interface
 static DB_decoder_t plugin = {
-    .plugin.api_vmajor = 1,
-    .plugin.api_vminor = 0,
+    DDB_PLUGIN_SET_API_VERSION
     .plugin.version_major = 1,
     .plugin.version_minor = 2,
     .plugin.type = DB_PLUGIN_DECODER,
