@@ -24,6 +24,7 @@
 #  include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <unistd.h>
@@ -46,115 +47,11 @@
 extern GtkWidget *mainwin;
 extern DB_functions_t *deadbeef;
 
-static gboolean
-file_filter_func (const GtkFileFilterInfo *filter_info, gpointer data) {
-    // get ext
-    const char *p = strrchr (filter_info->filename, '.');
-    if (!p) {
-        return FALSE;
-    }
-    p++;
-
-    // get beginning of fname
-    const char *fn = strrchr (filter_info->filename, '/');
-    if (!fn) {
-        fn = filter_info->filename;
-    }
-    else {
-        fn++;
-    }
-
-
-    DB_decoder_t **codecs = deadbeef->plug_get_decoder_list ();
-    for (int i = 0; codecs[i]; i++) {
-        if (codecs[i]->exts && codecs[i]->insert) {
-            const char **exts = codecs[i]->exts;
-            for (int e = 0; exts[e]; e++) {
-                if (!strcasecmp (exts[e], p)) {
-                    return TRUE;
-                }
-            }
-        }
-        if (codecs[i]->prefixes && codecs[i]->insert) {
-            const char **prefixes = codecs[i]->prefixes;
-            for (int e = 0; prefixes[e]; e++) {
-                if (!strncasecmp (prefixes[e], fn, strlen(prefixes[e])) && *(fn + strlen (prefixes[e])) == '.') {
-                    return TRUE;
-                }
-            }
-        }
-    }
-#if 0
-    if (!strcasecmp (p, "pls")) {
-        return TRUE;
-    }
-    if (!strcasecmp (p, "m3u")) {
-        return TRUE;
-    }
-#endif
-
-    // test container (vfs) formats
-    DB_vfs_t **vfsplugs = deadbeef->plug_get_vfs_list ();
-    for (int i = 0; vfsplugs[i]; i++) {
-        if (vfsplugs[i]->is_container) {
-            if (vfsplugs[i]->is_container (filter_info->filename)) {
-                return TRUE;
-            }
-        }
-    }
-
-    return FALSE;
-}
-
-static GtkFileFilter *
-set_file_filter (GtkWidget *dlg, const char *name) {
-    if (!name) {
-        name = _("Supported sound formats");
-    }
-
-    GtkFileFilter* flt;
-    flt = gtk_file_filter_new ();
-    gtk_file_filter_set_name (flt, name);
-
-    gtk_file_filter_add_custom (flt, GTK_FILE_FILTER_FILENAME, file_filter_func, NULL, NULL);
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dlg), flt);
-    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dlg), flt);
-    flt = gtk_file_filter_new ();
-    gtk_file_filter_set_name (flt, _("All files (*)"));
-    gtk_file_filter_add_pattern (flt, "*");
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dlg), flt);
-    return flt;
-}
-
 gboolean
 action_open_files_handler_cb (void *userdata) {
-    GtkWidget *dlg = gtk_file_chooser_dialog_new (_("Open file(s)..."), GTK_WINDOW (mainwin), GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
-
-    set_file_filter (dlg, NULL);
-
-    gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dlg), TRUE);
-    // restore folder
-    deadbeef->conf_lock ();
-    gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dlg), deadbeef->conf_get_str_fast ("filechooser.lastdir", ""));
-    deadbeef->conf_unlock ();
-    int response = gtk_dialog_run (GTK_DIALOG (dlg));
-    // store folder
-    gchar *folder = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dlg));
-    if (folder) {
-        deadbeef->conf_set_str ("filechooser.lastdir", folder);
-        g_free (folder);
-    }
-    if (response == GTK_RESPONSE_OK)
-    {
-        deadbeef->pl_clear ();
-        GSList *lst = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (dlg));
-        gtk_widget_destroy (dlg);
-        if (lst) {
-            gtkui_open_files (lst);
-        }
-    }
-    else {
-        gtk_widget_destroy (dlg);
+    GSList *lst = show_file_chooser(_("Open file(s)..."), GTKUI_FILECHOOSER_OPENFILE, TRUE);
+    if (lst) {
+        gtkui_open_files (lst);
     }
     return FALSE;
 }
@@ -167,33 +64,9 @@ action_open_files_handler (struct DB_plugin_action_s *action, int ctx) {
 
 gboolean
 action_add_files_handler_cb (void *user_data) {
-    GtkWidget *dlg = gtk_file_chooser_dialog_new (_("Add file(s) to playlist..."), GTK_WINDOW (mainwin), GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
-
-    set_file_filter (dlg, NULL);
-
-    gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dlg), TRUE);
-
-    // restore folder
-    deadbeef->conf_lock ();
-    gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dlg), deadbeef->conf_get_str_fast ("filechooser.lastdir", ""));
-    deadbeef->conf_unlock ();
-    int response = gtk_dialog_run (GTK_DIALOG (dlg));
-    // store folder
-    gchar *folder = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dlg));
-    if (folder) {
-        deadbeef->conf_set_str ("filechooser.lastdir", folder);
-        g_free (folder);
-    }
-    if (response == GTK_RESPONSE_OK)
-    {
-        GSList *lst = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (dlg));
-        gtk_widget_destroy (dlg);
-        if (lst) {
-            gtkui_add_files (lst);
-        }
-    }
-    else {
-        gtk_widget_destroy (dlg);
+    GSList *lst = show_file_chooser(_("Add file(s) to playlist..."), GTKUI_FILECHOOSER_OPENFILE, TRUE);
+    if (lst) {
+        gtkui_add_files (lst);
     }
     return FALSE;
 }
@@ -204,56 +77,11 @@ action_add_files_handler (struct DB_plugin_action_s *action, int ctx) {
     return 0;
 }
 
-static void
-on_follow_symlinks_toggled         (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-    deadbeef->conf_set_int ("add_folders_follow_symlinks", gtk_toggle_button_get_active (togglebutton));
-}
-
 gboolean
 action_add_folders_handler_cb (void *user_data) {
-    GtkWidget *dlg = gtk_file_chooser_dialog_new (_("Add folder(s) to playlist..."), GTK_WINDOW (mainwin), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
-
-    GtkWidget *box = gtk_hbox_new (FALSE, 8);
-    gtk_widget_show (box);
-
-    GtkWidget *check = gtk_check_button_new_with_mnemonic (_("Follow symlinks"));
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), deadbeef->conf_get_int ("add_folders_follow_symlinks", 0));
-    g_signal_connect ((gpointer) check, "toggled",
-            G_CALLBACK (on_follow_symlinks_toggled),
-            NULL);
-    gtk_widget_show (check);
-    gtk_box_pack_start (GTK_BOX (box), check, FALSE, FALSE, 0);
-
-    gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dlg), box);
-
-// gtk devs broke this in 3.6 - thanks guys
-//  set_file_filter (dlg, NULL);
-
-    gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dlg), TRUE);
-    // restore folder
-    deadbeef->conf_lock ();
-    gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dlg), deadbeef->conf_get_str_fast ("filechooser.lastdir", ""));
-    deadbeef->conf_unlock ();
-    int response = gtk_dialog_run (GTK_DIALOG (dlg));
-    // store folder
-    gchar *folder = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dlg));
-    if (folder) {
-        deadbeef->conf_set_str ("filechooser.lastdir", folder);
-        g_free (folder);
-    }
-    if (response == GTK_RESPONSE_OK)
-    {
-        //gchar *folder = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
-        GSList *lst = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (dlg));
-        gtk_widget_destroy (dlg);
-        if (lst) {
-            gtkui_add_dirs (lst);
-        }
-    }
-    else {
-        gtk_widget_destroy (dlg);
+    GSList *lst = show_file_chooser(_("Add folder(s) to playlist..."), GTKUI_FILECHOOSER_OPENFOLDER, TRUE);
+    if (lst) {
+        gtkui_add_dirs (lst);
     }
     return FALSE;
 }
@@ -289,11 +117,7 @@ action_deselect_all_handler_cb (void *user_data) {
         it = next;
     }
     deadbeef->pl_unlock ();
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
-    DdbListview *pl = DDB_LISTVIEW (lookup_widget (searchwin, "searchlist"));
-    if (pl) {
-        ddb_listview_refresh (pl, DDB_REFRESH_LIST);
-    }
+    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_SELECTION, 0);
     return FALSE;
 }
 
@@ -306,11 +130,7 @@ action_deselect_all_handler (struct DB_plugin_action_s *action, int ctx) {
 gboolean
 action_select_all_handler_cb (void *user_data) {
     deadbeef->pl_select_all ();
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
-    DdbListview *pl = DDB_LISTVIEW (lookup_widget (searchwin, "searchlist"));
-    if (pl) {
-        ddb_listview_refresh (pl, DDB_REFRESH_LIST);
-    }
+    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_SELECTION, 0);
     return FALSE;
 }
 
@@ -430,6 +250,7 @@ action_add_location_handler_cb (void *user_data) {
 #endif
 
     gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
+    gtk_window_set_transient_for (GTK_WINDOW (dlg), GTK_WINDOW (mainwin));
     int res = gtk_dialog_run (GTK_DIALOG (dlg));
     if (res == GTK_RESPONSE_OK) {
         GtkEntry *entry = GTK_ENTRY (lookup_widget (dlg, "addlocation_entry"));
@@ -449,7 +270,7 @@ action_add_location_handler_cb (void *user_data) {
                         deadbeef->pl_item_unref (tail);
                     }
                     deadbeef->plt_add_files_end (plt, 0);
-                    playlist_refresh ();
+                    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
                 }
                 if (plt) {
                     deadbeef->plt_unref (plt);
@@ -472,7 +293,7 @@ static GtkWidget *helpwindow;
 gboolean
 action_show_help_handler_cb (void *user_data) {
     char fname[PATH_MAX];
-    snprintf (fname, sizeof (fname), "%s/%s", deadbeef->get_doc_dir (), _("help.txt"));
+    snprintf (fname, sizeof (fname), "%s/%s", deadbeef->get_system_dir(DDB_SYS_DIR_DOC), _("help.txt"));
     gtkui_show_info_window (fname, _("Help"), &helpwindow);
     return FALSE;
 }
@@ -498,13 +319,19 @@ action_remove_from_playlist_handler (DB_plugin_action_t *act, int ctx) {
             }
             deadbeef->plt_save_config (plt);
             deadbeef->plt_unref (plt);
-            deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+            deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
         }
     }
     else if (ctx == DDB_ACTION_CTX_PLAYLIST) {
-        deadbeef->pl_clear ();
-        deadbeef->pl_save_current ();
-        deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+        ddb_playlist_t *plt_curr = deadbeef->plt_get_curr ();
+        ddb_playlist_t *plt = deadbeef->action_get_playlist ();
+        deadbeef->plt_clear (plt);
+        deadbeef->plt_save_config (plt);
+        if (plt == plt_curr) {
+            deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
+        }
+        deadbeef->plt_unref (plt);
+        deadbeef->plt_unref (plt_curr);
     }
     else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
         int success = 0;
@@ -517,7 +344,7 @@ action_remove_from_playlist_handler (DB_plugin_action_t *act, int ctx) {
                 if (idx != -1) {
                     deadbeef->plt_remove_item (plt, it);
                     deadbeef->pl_save_current ();
-                    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+                    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
                 }
                 deadbeef->plt_unref (plt);
             }
@@ -565,11 +392,11 @@ action_delete_from_disk_handler_cb (void *data) {
     if (ctx == DDB_ACTION_CTX_SELECTION) {
         DB_playItem_t *it = deadbeef->plt_get_first (plt, PL_MAIN);
         while (it) {
+            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
             const char *uri = deadbeef->pl_find_meta (it, ":URI");
             if (deadbeef->pl_is_selected (it) && deadbeef->is_local_file (uri)) {
                 delete_and_remove_track (uri, plt, it);
             }
-            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
             deadbeef->pl_item_unref (it);
             it = next;
         }
@@ -616,7 +443,7 @@ action_delete_from_disk_handler_cb (void *data) {
     deadbeef->pl_unlock ();
     deadbeef->plt_unref (plt);
 
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
     return FALSE;
 }
 
@@ -626,21 +453,32 @@ action_delete_from_disk_handler (DB_plugin_action_t *act, int ctx) {
     return 0;
 }
 
+typedef struct {
+    int ctx;
+    ddb_playlist_t *plt;
+} trkproperties_action_ctx_t;
+
 gboolean
 action_show_track_properties_handler_cb (void *data) {
-    show_track_properties_dlg ((intptr_t)data);
+    trkproperties_action_ctx_t *ctx = data;
+    show_track_properties_dlg (ctx->ctx, ctx->plt);
+    deadbeef->plt_unref (ctx->plt);
+    free (data);
     return FALSE;
 }
 
 int
 action_show_track_properties_handler (DB_plugin_action_t *act, int ctx) {
-    gdk_threads_add_idle (action_show_track_properties_handler_cb, (void *)(intptr_t)ctx);
+    trkproperties_action_ctx_t *data = calloc (1, sizeof (trkproperties_action_ctx_t));
+    data->ctx = ctx;
+    data->plt = deadbeef->action_get_playlist ();
+    gdk_threads_add_idle (action_show_track_properties_handler_cb, data);
     return 0;
 }
 
 gboolean
 action_find_handler_cb (void *data) {
-    search_start ();       
+    search_start ();
     return FALSE;
 }
 
@@ -678,30 +516,6 @@ action_cursor_follows_playback_handler (DB_plugin_action_t *act, int ctx) {
     return 0;
 }
 
-static gboolean
-playlist_filter_func (const GtkFileFilterInfo *filter_info, gpointer data) {
-    // get ext
-    const char *p = strrchr (filter_info->filename, '.');
-    if (!p) {
-        return FALSE;
-    }
-    p++;
-    DB_playlist_t **plug = deadbeef->plug_get_playlist_list ();
-    for (int i = 0; plug[i]; i++) {
-        if (plug[i]->extensions && plug[i]->load) {
-            const char **exts = plug[i]->extensions;
-            if (exts) {
-                for (int e = 0; exts[e]; e++) {
-                    if (!strcasecmp (exts[e], p)) {
-                        return TRUE;
-                    }
-                }
-            }
-        }
-    }
-    return FALSE;
-}
-
 static void
 load_playlist_thread (void *data) {
     char *fname = data;
@@ -717,49 +531,21 @@ load_playlist_thread (void *data) {
         deadbeef->plt_unref (plt);
     }
     g_free (fname);
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
 }
 
-
 gboolean
 action_load_playlist_handler_cb (void *data) {
-    GtkWidget *dlg = gtk_file_chooser_dialog_new (_("Load Playlist"), GTK_WINDOW (mainwin), GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
+    GSList *lst = show_file_chooser(_("Load Playlist"), GTKUI_FILECHOOSER_LOADPLAYLIST, FALSE);
 
-    // restore folder
-    deadbeef->conf_lock ();
-    gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dlg), deadbeef->conf_get_str_fast ("filechooser.playlist.lastdir", ""));
-    deadbeef->conf_unlock ();
-
-    GtkFileFilter* flt;
-    flt = gtk_file_filter_new ();
-    gtk_file_filter_set_name (flt, "Supported playlist formats");
-    gtk_file_filter_add_custom (flt, GTK_FILE_FILTER_FILENAME, playlist_filter_func, NULL, NULL);
-    gtk_file_filter_add_pattern (flt, "*.dbpl");
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dlg), flt);
-    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dlg), flt);
-    flt = gtk_file_filter_new ();
-    gtk_file_filter_set_name (flt, _("Other files (*)"));
-    gtk_file_filter_add_pattern (flt, "*");
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dlg), flt);
-    
-    int res = gtk_dialog_run (GTK_DIALOG (dlg));
-    // store folder
-    gchar *folder = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dlg));
-    if (folder) {
-        deadbeef->conf_set_str ("filechooser.playlist.lastdir", folder);
-        g_free (folder);
-    }
-    if (res == GTK_RESPONSE_OK)
+    if (lst)
     {
-        gchar *fname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
-        gtk_widget_destroy (dlg);
+        gchar *fname = g_slist_nth_data (lst, 0);
         if (fname) {
             uintptr_t tid = deadbeef->thread_start (load_playlist_thread, fname);
             deadbeef->thread_detach (tid);
         }
-    }
-    else {
-        gtk_widget_destroy (dlg);
+        g_slist_free(lst);
     }
     return FALSE;
 }
@@ -772,48 +558,12 @@ action_load_playlist_handler (DB_plugin_action_t *act, int ctx) {
 
 gboolean
 action_save_playlist_handler_cb (void *data) {
-    GtkWidget *dlg = gtk_file_chooser_dialog_new (_("Save Playlist As"), GTK_WINDOW (mainwin), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_OK, NULL);
+    GSList *lst = show_file_chooser(_("Save Playlist As"), GTKUI_FILECHOOSER_SAVEPLAYLIST, FALSE);
 
-    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dlg), TRUE);
-    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dlg), "untitled.dbpl");
-    // restore folder
-    deadbeef->conf_lock ();
-    gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dlg), deadbeef->conf_get_str_fast ("filechooser.playlist.lastdir", ""));
-    deadbeef->conf_unlock ();
 
-    GtkFileFilter* flt;
-    flt = gtk_file_filter_new ();
-    gtk_file_filter_set_name (flt, _("DeaDBeeF playlist files (*.dbpl)"));
-    gtk_file_filter_add_pattern (flt, "*.dbpl");
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dlg), flt);
-    DB_playlist_t **plug = deadbeef->plug_get_playlist_list ();
-    for (int i = 0; plug[i]; i++) {
-        if (plug[i]->extensions && plug[i]->load) {
-            const char **exts = plug[i]->extensions;
-            if (exts && plug[i]->save) {
-                for (int e = 0; exts[e]; e++) {
-                    char s[100];
-                    flt = gtk_file_filter_new ();
-                    gtk_file_filter_set_name (flt, exts[e]);
-                    snprintf (s, sizeof (s), "*.%s", exts[e]);
-                    gtk_file_filter_add_pattern (flt, s);
-                    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dlg), flt);
-                }
-            }
-        }
-    }
-
-    int res = gtk_dialog_run (GTK_DIALOG (dlg));
-    // store folder
-    gchar *folder = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dlg));
-    if (folder) {
-        deadbeef->conf_set_str ("filechooser.playlist.lastdir", folder);
-        g_free (folder);
-    }
-    if (res == GTK_RESPONSE_OK)
+    if (lst)
     {
-        gchar *fname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
-        gtk_widget_destroy (dlg);
+        gchar *fname = g_slist_nth_data (lst, 0);
 
         if (fname) {
             ddb_playlist_t *plt = deadbeef->plt_get_curr ();
@@ -825,11 +575,9 @@ action_save_playlist_handler_cb (void *data) {
                 deadbeef->plt_unref (plt);
             }
             g_free (fname);
+            g_slist_free(lst);
         }
     }
-    else {
-        gtk_widget_destroy (dlg);
-    }
     return FALSE;
 }
 
@@ -904,13 +652,13 @@ gboolean
 action_sort_custom_handler_cb (void *data) {
     GtkWidget *dlg = create_sortbydlg ();
     gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
-    
+
     GtkComboBox *combo = GTK_COMBO_BOX (lookup_widget (dlg, "sortorder"));
     GtkEntry *entry = GTK_ENTRY (lookup_widget (dlg, "sortfmt"));
-    
+
     gtk_combo_box_set_active (combo, deadbeef->conf_get_int ("gtkui.sortby_order", 0));
     deadbeef->conf_lock ();
-    gtk_entry_set_text (entry, deadbeef->conf_get_str_fast ("gtkui.sortby_fmt", ""));
+    gtk_entry_set_text (entry, deadbeef->conf_get_str_fast ("gtkui.sortby_fmt_v2", ""));
     deadbeef->conf_unlock ();
 
     int r = gtk_dialog_run (GTK_DIALOG (dlg));
@@ -922,14 +670,14 @@ action_sort_custom_handler_cb (void *data) {
         const char *fmt = gtk_entry_get_text (entry);
 
         deadbeef->conf_set_int ("gtkui.sortby_order", order);
-        deadbeef->conf_set_str ("gtkui.sortby_fmt", fmt);
+        deadbeef->conf_set_str ("gtkui.sortby_fmt_v2", fmt);
 
         ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-        deadbeef->plt_sort (plt, PL_MAIN, -1, fmt, order == 0 ? DDB_SORT_ASCENDING : DDB_SORT_DESCENDING);
+        deadbeef->plt_sort_v2 (plt, PL_MAIN, -1, fmt, order == 0 ? DDB_SORT_ASCENDING : DDB_SORT_DESCENDING);
         deadbeef->plt_save_config (plt);
         deadbeef->plt_unref (plt);
 
-        deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+        deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
     }
 
     gtk_widget_destroy (dlg);
@@ -947,7 +695,7 @@ int
 action_crop_selected_handler (DB_plugin_action_t *act, int ctx) {
     deadbeef->pl_crop_selected ();
     deadbeef->pl_save_current ();
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
     return 0;
 }
 
@@ -1130,3 +878,15 @@ action_playback_loop_cycle_handler(DB_plugin_action_t *act, int ctx) {
     g_idle_add (action_playback_loop_cycle_handler_cb, NULL);
     return 0;
 }
+
+gboolean
+action_toggle_logwindow_handler_cb (void *data) {
+    gtkui_toggle_log_window();
+    return FALSE;
+}
+
+int
+action_toggle_logwindow_handler(DB_plugin_action_t *act, int ctx) {
+    g_idle_add (action_toggle_logwindow_handler_cb, NULL);
+    return 0;
+}
