@@ -1,21 +1,26 @@
 /*
-    DeaDBeeF - The Ultimate Music Player
-    Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>
+    DeaDBeeF -- the music player
+    Copyright (C) 2009-2015 Alexey Yakovenko and other contributors
 
-    This program is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License
-    as published by the Free Software Foundation; either version 2
-    of the License, or (at your option) any later version.
-    
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-    
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+    This software is provided 'as-is', without any express or implied
+    warranty.  In no event will the authors be held liable for any damages
+    arising from the use of this software.
+
+    Permission is granted to anyone to use this software for any purpose,
+    including commercial applications, and to alter it and redistribute it
+    freely, subject to the following restrictions:
+
+    1. The origin of this software must not be misrepresented; you must not
+     claim that you wrote the original software. If you use this software
+     in a product, an acknowledgment in the product documentation would be
+     appreciated but is not required.
+
+    2. Altered source versions must be plainly marked as such, and must not be
+     misrepresented as being the original software.
+
+    3. This notice may not be removed or altered from any source distribution.
 */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -210,6 +215,19 @@ ddb_button_from_gtk_response (int response) {
     return -1;
 }
 
+static int
+backout_pack_level(int ncurr, int *pack)
+{
+    if (ncurr > 0) {
+        pack[ncurr]--;
+        if (pack[ncurr] < 0) {
+            ncurr--;
+            ncurr = backout_pack_level(ncurr, pack);
+        }
+    }
+    return ncurr;
+}
+
 int
 gtkui_run_dialog (GtkWidget *parentwin, ddb_dialog_t *conf, uint32_t buttons, int (*callback)(int button, void *ctx), void *ctx) {
     if (!parentwin) {
@@ -277,12 +295,7 @@ gtkui_run_dialog (GtkWidget *parentwin, ddb_dialog_t *conf, uint32_t buttons, in
             break;
         }
 
-        if (ncurr > 0) {
-            pack[ncurr]--;
-            if (pack[ncurr] < 0) {
-                ncurr--;
-            }
-        }
+        ncurr = backout_pack_level(ncurr, pack);
 
         char type[MAX_TOKEN];
         script = gettoken_warn_eof (script, type);
@@ -298,7 +311,7 @@ gtkui_run_dialog (GtkWidget *parentwin, ddb_dialog_t *conf, uint32_t buttons, in
             }
             pack[ncurr] = n;
 
-            int vert = 0;
+            int vert = !strncmp (type, "vbox[", 5);
             int hmg = FALSE;
             int fill = FALSE;
             int expand = FALSE;
@@ -335,8 +348,8 @@ gtkui_run_dialog (GtkWidget *parentwin, ddb_dialog_t *conf, uint32_t buttons, in
                 }
             }
 
-            widgets[ncurr] = vert ? gtk_vbox_new (TRUE, spacing) : gtk_hbox_new (TRUE, spacing);
-            gtk_widget_set_size_request (widgets[ncurr], -1, height);
+            widgets[ncurr] = vert ? gtk_vbox_new (hmg, spacing) : gtk_hbox_new (hmg, spacing);
+            gtk_widget_set_size_request (widgets[ncurr], vert ? height : -1, vert ? -1 : height);
             gtk_widget_show (widgets[ncurr]);
             gtk_box_pack_start (GTK_BOX(widgets[ncurr-1]), widgets[ncurr], fill, expand, border);
             continue;
@@ -476,9 +489,12 @@ gtkui_run_dialog (GtkWidget *parentwin, ddb_dialog_t *conf, uint32_t buttons, in
                     gtk_range_set_inverted (GTK_RANGE (prop), TRUE);
                 }
                 gtk_range_set_value (GTK_RANGE (prop), (gdouble)atof (value));
-                gtk_scale_set_value_pos (GTK_SCALE (prop), GTK_POS_RIGHT);
+                gtk_scale_set_value_pos (GTK_SCALE (prop), vertical?GTK_POS_BOTTOM:GTK_POS_RIGHT);
             }
             label = gtk_label_new (_(labeltext));
+            if (vertical) {
+                gtk_label_set_angle(GTK_LABEL(label), 270);
+            }
             gtk_widget_show (label);
             g_signal_connect (G_OBJECT (prop), "value-changed", G_CALLBACK (prop_changed), win);
             gtk_widget_show (prop);
@@ -509,7 +525,7 @@ gtkui_run_dialog (GtkWidget *parentwin, ddb_dialog_t *conf, uint32_t buttons, in
             g_object_set_data (G_OBJECT (win), key, prop);
         }
         if (cont) {
-            gtk_box_pack_start (GTK_BOX (widgets[ncurr]), cont, FALSE, FALSE, 0);
+            gtk_box_pack_start (GTK_BOX (widgets[ncurr]), cont, FALSE, TRUE, 0);
         }
     }
 
