@@ -25,7 +25,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#ifndef __APPLE__
+#if defined(__MINGW32__) || defined(__APPLE__)
+#define NO_XLIB_H
+#endif
+#ifndef NO_XLIB_H
 #include <X11/Xlib.h>
 #endif
 #include <ctype.h>
@@ -44,7 +47,7 @@
 static DB_hotkeys_plugin_t plugin;
 DB_functions_t *deadbeef;
 
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
 static int finished;
 static Display *disp;
 static intptr_t loop_tid;
@@ -56,7 +59,7 @@ static int need_reset = 0;
 typedef struct {
     const char *name;
     int keysym;
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
     int keycode; // after mapping
 #endif
 } xkey_t;
@@ -69,7 +72,7 @@ static xkey_t keys[] = {
 
 typedef struct command_s {
     int keycode;
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
     int x11_keycode;
 #endif
     int modifier;
@@ -81,7 +84,7 @@ typedef struct command_s {
 static command_t commands [MAX_COMMAND_COUNT];
 static int command_count = 0;
 
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
 static void
 init_mapped_keycodes (Display *disp, Atom *syms, int first_kk, int last_kk, int ks_per_kk) {
     int i, ks;
@@ -227,7 +230,7 @@ find_action_by_name (const char *command) {
     return actions;
 }
 
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
 static int
 get_x11_keycode (const char *name, Atom *syms, int first_kk, int last_kk, int ks_per_kk) {
     int i, ks;
@@ -349,7 +352,7 @@ read_config (void) {
                 else {
                     // lookup name table
                     cmd_entry->keycode = get_keycode (p);
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
                     cmd_entry->x11_keycode = get_x11_keycode (p, syms, first_kk, last_kk, ks_per_kk);
                     trace ("%s: kc=%d, xkc=%d\n", p, cmd_entry->keycode, cmd_entry->x11_keycode);
 #endif
@@ -373,7 +376,7 @@ read_config (void) {
 out:
         item = deadbeef->conf_find ("hotkey.", item);
     }
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
     XFree (syms);
     int i;
     // need to grab it here to prevent gdk_x_error from being called while we're
@@ -414,7 +417,7 @@ hotkeys_load (DB_functions_t *api) {
 static void
 cleanup () {
     command_count = 0;
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
     if (disp) {
         XCloseDisplay (disp);
         disp = NULL;
@@ -422,7 +425,7 @@ cleanup () {
 #endif
 }
 
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
 static int
 x_err_handler (Display *d, XErrorEvent *evt) {
 #if 0
@@ -502,7 +505,7 @@ hotkeys_event_loop (void *unused) {
 
 static int
 hotkeys_connect (void) {
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
     finished = 0;
     loop_tid = 0;
     disp = XOpenDisplay (NULL);
@@ -532,7 +535,7 @@ hotkeys_connect (void) {
 
 static int
 hotkeys_disconnect (void) {
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
     if (loop_tid) {
         finished = 1;
         deadbeef->thread_join (loop_tid);
@@ -579,7 +582,7 @@ hotkeys_get_action_for_keycombo (int key, int mods, int isglobal, int *ctx) {
 
 void
 hotkeys_reset (void) {
-#ifndef __APPLE__
+#ifndef NO_XLIB_H
     need_reset = 1;
     trace ("hotkeys: reset flagged\n");
 #endif
@@ -743,15 +746,55 @@ action_seek_1p_backward_cb (struct DB_plugin_action_s *action, int ctx) {
     return 0;
 }
 
+static int
+seek_sec (float sec) {
+    deadbeef->pl_lock ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    if (it) {
+        float dur = deadbeef->pl_get_item_duration (it);
+        if (dur > 0) {
+            float pos = deadbeef->streamer_get_playpos ();
+            pos += sec;
+            if (pos < 0) {
+                pos = 0;
+            }
+            deadbeef->sendmessage (DB_EV_SEEK, 0, pos * 1000, 0);
+        }
+        deadbeef->pl_item_unref (it);
+    }
+    deadbeef->pl_unlock ();
+    return 0;
+}
+
+int
+action_seek_1s_forward_cb (struct DB_plugin_action_s *action, int ctx) {
+    return seek_sec (1.f);
+}
+
+int
+action_seek_1s_backward_cb (struct DB_plugin_action_s *action, int ctx) {
+    return seek_sec (-1.f);
+}
+
+int
+action_seek_5s_forward_cb (struct DB_plugin_action_s *action, int ctx) {
+    return seek_sec (5.f);
+}
+
+int
+action_seek_5s_backward_cb (struct DB_plugin_action_s *action, int ctx) {
+    return seek_sec (-5.f);
+}
+
 int
 action_volume_up_cb (struct DB_plugin_action_s *action, int ctx) {
-    deadbeef->volume_set_db (deadbeef->volume_get_db () + 2);
+    deadbeef->volume_set_db (deadbeef->volume_get_db () + 1);
     return 0;
 }
 
 int
 action_volume_down_cb (struct DB_plugin_action_s *action, int ctx) {
-    deadbeef->volume_set_db (deadbeef->volume_get_db () - 2);
+    deadbeef->volume_set_db (deadbeef->volume_get_db () - 1);
     return 0;
 }
 
@@ -789,12 +832,76 @@ static DB_plugin_action_t action_jump_to_current = {
     .next = &action_reload_metadata
 };
 
+static DB_plugin_action_t action_skip_to_prev_genre = {
+    .title = "Playback/Skip to/Previous genre",
+    .name = "skip_to_prev_genre",
+    .flags = DB_ACTION_COMMON | DB_ACTION_ADD_MENU,
+    .callback2 = action_skip_to_prev_genre_handler,
+    .next = &action_jump_to_current
+};
+
+static DB_plugin_action_t action_skip_to_prev_composer = {
+    .title = "Playback/Skip to/Previous composer",
+    .name = "skip_to_prev_composer",
+    .flags = DB_ACTION_COMMON | DB_ACTION_ADD_MENU,
+    .callback2 = action_skip_to_prev_composer_handler,
+    .next = &action_skip_to_prev_genre
+};
+
+static DB_plugin_action_t action_skip_to_prev_artist = {
+    .title = "Playback/Skip to/Previous artist",
+    .name = "skip_to_prev_artist",
+    .flags = DB_ACTION_COMMON | DB_ACTION_ADD_MENU,
+    .callback2 = action_skip_to_prev_artist_handler,
+    .next = &action_skip_to_prev_composer
+};
+
+static DB_plugin_action_t action_skip_to_prev_album = {
+    .title = "Playback/Skip to/Previous album",
+    .name = "skip_to_prev_album",
+    .flags = DB_ACTION_COMMON | DB_ACTION_ADD_MENU,
+    .callback2 = action_skip_to_prev_album_handler,
+    .next = &action_skip_to_prev_artist
+};
+
+static DB_plugin_action_t action_skip_to_next_genre = {
+    .title = "Playback/Skip to/Next genre",
+    .name = "skip_to_next_genre",
+    .flags = DB_ACTION_COMMON | DB_ACTION_ADD_MENU,
+    .callback2 = action_skip_to_next_genre_handler,
+    .next = &action_skip_to_prev_album
+};
+
+static DB_plugin_action_t action_skip_to_next_composer = {
+    .title = "Playback/Skip to/Next composer",
+    .name = "skip_to_next_composer",
+    .flags = DB_ACTION_COMMON | DB_ACTION_ADD_MENU,
+    .callback2 = action_skip_to_next_composer_handler,
+    .next = &action_skip_to_next_genre
+};
+
+static DB_plugin_action_t action_skip_to_next_artist = {
+    .title = "Playback/Skip to/Next artist",
+    .name = "skip_to_next_artist",
+    .flags = DB_ACTION_COMMON | DB_ACTION_ADD_MENU,
+    .callback2 = action_skip_to_next_artist_handler,
+    .next = &action_skip_to_next_composer
+};
+
+static DB_plugin_action_t action_skip_to_next_album = {
+    .title = "Playback/Skip to/Next album",
+    .name = "skip_to_next_album",
+    .flags = DB_ACTION_COMMON | DB_ACTION_ADD_MENU,
+    .callback2 = action_skip_to_next_album_handler,
+    .next = &action_skip_to_next_artist
+};
+
 static DB_plugin_action_t action_next_playlist = {
     .title = "Next Playlist",
     .name = "next_playlist",
     .flags = DB_ACTION_COMMON,
     .callback2 = action_next_playlist_handler,
-    .next = &action_jump_to_current
+    .next = &action_skip_to_next_album
 };
 
 static DB_plugin_action_t action_prev_playlist = {
@@ -966,12 +1073,36 @@ static DB_plugin_action_t action_add_to_playqueue = {
     .next = &action_remove_from_playqueue
 };
 
+static DB_plugin_action_t action_toggle_in_playqueue = {
+    .title = "Playback/Toggle In Playback Queue",
+    .name = "toggle_in_playback_queue",
+    .flags = DB_ACTION_MULTIPLE_TRACKS | DB_ACTION_EXCLUDE_FROM_CTX_PLAYLIST,
+    .callback2 = action_toggle_in_playqueue_handler,
+    .next = &action_add_to_playqueue
+};
+
+static DB_plugin_action_t action_move_tracks_down = {
+    .title = "Move/Move Tracks Down",
+    .name = "move_tracks_down",
+    .flags = DB_ACTION_MULTIPLE_TRACKS | DB_ACTION_EXCLUDE_FROM_CTX_PLAYLIST,
+    .callback2 = action_move_tracks_down_handler,
+    .next = &action_toggle_in_playqueue
+};
+
+static DB_plugin_action_t action_move_tracks_up = {
+    .title = "Move/Move Tracks Up",
+    .name = "move_tracks_up",
+    .flags = DB_ACTION_MULTIPLE_TRACKS | DB_ACTION_EXCLUDE_FROM_CTX_PLAYLIST,
+    .callback2 = action_move_tracks_up_handler,
+    .next = &action_move_tracks_down
+};
+
 static DB_plugin_action_t action_toggle_mute = {
     .title = "Playback/Toggle Mute",
     .name = "toggle_mute",
     .flags = DB_ACTION_COMMON,
     .callback2 = action_toggle_mute_handler,
-    .next = &action_add_to_playqueue
+    .next = &action_move_tracks_up
 };
 
 static DB_plugin_action_t action_play = {
@@ -1030,12 +1161,45 @@ static DB_plugin_action_t action_play_random = {
     .next = &action_play_pause
 };
 
+static DB_plugin_action_t action_seek_1s_forward = {
+    .title = "Playback/Seek 1s Forward",
+    .name = "seek_1s_fwd",
+    .flags = DB_ACTION_COMMON,
+    .callback2 = action_seek_1s_forward_cb,
+    .next = &action_play_random
+};
+
+static DB_plugin_action_t action_seek_1s_backward = {
+    .title = "Playback/Seek 1s Backward",
+    .name = "seek_1s_back",
+    .flags = DB_ACTION_COMMON,
+    .callback2 = action_seek_1s_backward_cb,
+    .next = &action_seek_1s_forward
+};
+
+static DB_plugin_action_t action_seek_5s_forward = {
+    .title = "Playback/Seek 5s Forward",
+    .name = "seek_5s_fwd",
+    .flags = DB_ACTION_COMMON,
+    .callback2 = action_seek_5s_forward_cb,
+    .next = &action_seek_1s_backward
+};
+
+static DB_plugin_action_t action_seek_5s_backward = {
+    .title = "Playback/Seek 5s Backward",
+    .name = "seek_5s_back",
+    .flags = DB_ACTION_COMMON,
+    .callback2 = action_seek_5s_backward_cb,
+    .next = &action_seek_5s_forward
+};
+
+
 static DB_plugin_action_t action_seek_1p_forward = {
     .title = "Playback/Seek 1% Forward",
     .name = "seek_1p_fwd",
     .flags = DB_ACTION_COMMON,
     .callback2 = action_seek_1p_forward_cb,
-    .next = &action_play_random
+    .next = &action_seek_5s_backward
 };
 
 static DB_plugin_action_t action_seek_1p_backward = {
@@ -1102,8 +1266,8 @@ hotkeys_get_actions (DB_playItem_t *it)
 
 // define plugin interface
 static DB_hotkeys_plugin_t plugin = {
-    .misc.plugin.api_vmajor = 1,
-    .misc.plugin.api_vminor = 5,
+    .misc.plugin.api_vmajor = DB_API_VERSION_MAJOR,
+    .misc.plugin.api_vminor = DB_API_VERSION_MINOR,
     .misc.plugin.version_major = 1,
     .misc.plugin.version_minor = 1,
     .misc.plugin.type = DB_PLUGIN_MISC,
